<?php
/**
 * security-header.php
 * 
 * Appends security headers to the active theme's functions.php
 * once, checking for a unique marker to prevent duplication.
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// (Optional) If needed, ensure WordPress is fully loaded:
if ( ! function_exists( 'get_stylesheet_directory' ) ) {
    require_once dirname(__DIR__, 3) . '/wp-load.php';
}

class Test_AppendToFunctions {
    private $functions_file;

    public function __construct() {
        // Path to the active (child) theme’s functions.php
        // If you want to target the parent theme, use get_template_directory()
        $this->functions_file = get_stylesheet_directory() . '/functions.php';

        // Attempt to append the code
        $this->append_code_to_functions();
    }

    private function append_code_to_functions() {
        // A unique marker to detect if we’ve already appended the snippet
        $marker = 'WP_KEEPUP_SECURITY_HEADER_MARKER';

        // This is the block of PHP code to append, including the marker
        $code_to_append = <<<'EOD'

// ========== WP KeepSoul Security Headers Start ==========
// WP_KEEPUP_SECURITY_HEADER_MARKER

// Remove WordPress version number
remove_action('wp_head', 'wp_generator');

// Remove version numbers from scripts and styles
function remove_version_from_assets($src) {
    return remove_query_arg('ver', $src);
}
add_filter('style_loader_src', 'remove_version_from_assets', 9999);
add_filter('script_loader_src', 'remove_version_from_assets', 9999);

// Disable REST API for unauthorized users
function restrict_rest_api_access($result) {
    if (!is_user_logged_in()) {
        return new WP_Error('rest_disabled', 'REST API restricted.', ['status' => 403]);
    }
    return $result;
}
add_filter('rest_authentication_errors', 'restrict_rest_api_access');

// Block XML-RPC methods to hide information
add_filter('xmlrpc_enabled', '__return_false');

// Generate CSP Nonce for scripts and styles
function set_csp_nonce() {
    global $csp_nonce;
    $csp_nonce = base64_encode(random_bytes(16));
    header("Content-Security-Policy: 
        default-src 'self'; 
        script-src 'self' 'nonce-$csp_nonce'; 
        style-src 'self' 'nonce-$csp_nonce'; 
        img-src 'self' data:; 
        frame-ancestors 'self'; 
        object-src 'none'; 
        base-uri 'self'; 
        form-action 'self'; 
        report-uri /csp-report-endpoint;
    ");
}
add_action('send_headers', 'set_csp_nonce');

// Add Subresource Integrity (SRI) for specific scripts
function add_sri_to_scripts($tag, $handle, $src) {
    $scripts_with_sri = [
        'jquery' => 'sha384-KyZXEAg3QhqLMpG8r+Knujsl5+5hb7t1pbC8YPvYmGkeTXriMfI/jxZ1rBRR9+W9',
    ];

    if (array_key_exists($handle, $scripts_with_sri)) {
        return str_replace('<script ', '<script integrity="' . $scripts_with_sri[$handle] . '" crossorigin="anonymous" ', $tag);
    }
    return $tag;
}
add_filter('script_loader_tag', 'add_sri_to_scripts', 10, 3);

// Additional security headers
add_action('send_headers', function() {
    // Referrer-Policy
    header('Referrer-Policy: strict-origin-when-cross-origin');

    // Permissions-Policy
    header('Permissions-Policy: geolocation=()');

    // Basic CSP directives (note: you already have a CSP via set_csp_nonce())
    header("Content-Security-Policy: default-src 'self'; object-src 'none'; frame-ancestors 'self';");

    // Strict-Transport-Security (HSTS)
    if (is_ssl()) {
        header("Strict-Transport-Security: max-age=31536000; includeSubDomains; preload");
    }

    // Remove X-Powered-By header
    header_remove('X-Powered-By');
});

// ========== WP KeepSoul Security Headers End ==========

EOD;

        // Check if the file exists and is writable
        if ( file_exists($this->functions_file) && is_writable($this->functions_file) ) {
            // Read the current content of the functions.php
            $content = file_get_contents($this->functions_file);

            // Check if the marker is already present
            if ( strpos($content, $marker) === false ) {
                // Append the code
                if ( file_put_contents($this->functions_file, $code_to_append, FILE_APPEND) !== false ) {
                   echo '<div class="notice notice-success">
    <p><strong>Security Header</strong> has been successfully appended!</p>
    <p>Please re-scan your site at <a href="https://securityheaders.com/" target="_blank">securityheaders.com</a> to verify.</p>
</div>';
      
                } else {
                    echo '<div class="notice notice-warning">
    <p><strong>Warning:</strong> Error writing security headers to <code>functions.php</code>. Please check file permissions and try again.</p>
</div>';

                }
            } else {
                // The marker (and thus the code) is already in functions.php
                echo '<div class="notice notice-error">
    <p><strong>Error:</strong> Security headers code already exists in your <code>functions.php</code>. No action taken.</p>
</div>';

            }
        } else {
            echo 'Error: functions.php not found or not writable.';
        }
    }
}

// Instantiate and run
new Test_AppendToFunctions();
